<?php
if(count(debug_backtrace())==0){
  //このファイルがrequireやinclude経由ではなく単独で呼ばれた場合
  header('location: ../');
  exit;
}
require(__DIR__.'/cmsModelBase.php');

class cmsModel extends cmsModelBase{
  //★ 初回「管理者(admin)」登録
  public function registAdminUser($pass){
    $sql='INSERT INTO
            tm_user(uname,userid,password,pdate,urdate)
            values(?,?,?,NOW(),NOW())';
    $this->execSQL($sql,['管理者','admin',hash('sha256',$pass.$this->_salt,false)]);
  }


  //★ 管理者パスワード更新
  public function updateAdminPassword($uid,$pass){
    $sql='UPDATE tm_user set password=?, pdate=NOW() where uid=?';
    $this->execSQL($sql,[hash('sha256',$pass.$this->_salt,false),$uid]);
  }


  //★ IDとパスワードからログインチェックを行う
  //戻り値：ユーザーID
  public function loginCheck($userid,$pass){
    $sql='SELECT uid FROM tm_user
            WHERE userid=? and password=?
         ';
    $result=$this->execSQL($sql,[$userid,hash('sha256',$pass.$this->_salt,false)]);
    if(count($result)==0){
      return 0;
    }else{
      //ログイン日時を更新する
      $sql='UPDATE tm_user set ldate=ndate,ndate=NOW()';
      $this->execSQL($sql,[]);
      $res=$result[0];
      return $res['uid'];
    }
  }


  //★ パスワード変更日、前回ログイン日
  public function getLoginPassDate(){
    if(array_key_exists('uid',$_SESSION)){
      $sql='SELECT pdate,ndate FROM tm_user where uid=?';
      $result=$this->execSQL($sql,[$_SESSION['uid']]);
      if(count($result)>0){
        $res=$result[0];
        return [$res['pdate'],$res['ndate']];
      }else{
        return ['',''];
      }
    }else{
      session_destroy();
      header('Location: ./');
      return false;
    }
  }

  //★ ログインしているかどうか（セッション変数 uidから調べる）
  public function isLogin(){
    $this->sessionSetParam();
    session_start();
    header('Expires: -1');
    header('Cache-Control:');
    header('Pragma:');
    if(array_key_exists('uid',$_SESSION)&&$_SESSION['uid']>0){
      return $_SESSION['uid'];
    }else{
      session_destroy();
      header('Location: ./');
      return false;
    }
  }

  //PHP標準セッションにフラグを設定する
  public function sessionSetParam(){
    session_set_cookie_params(
      0,//セッション
      substr($_SERVER["PHP_SELF"],0,strlen($_SERVER["PHP_SELF"])-strlen(basename($_SERVER["PHP_SELF"]))),//Path
      "",            //Domain
      empty($_SERVER["HTTPS"])?false:true, //Secure属性(HTTPSのみにしたい場合はTrue)
      true           //HttpOnly属性(JavaScriptからはアクセス不可)
    );
  }

  //★ 大カテゴリ一覧を取得する
  public function getCategory(){
    $sql='select * from tm_category where pcid=0 order by corder desc';
    return $this->execSQL($sql);
  }
  //★★★BO用 大カテゴリと、小カテゴリを内部結合したリストを出力する
  public function getCateSubCateList(){
    $sql='
      select
        c.cid cid,
        c.cname,
        sc.cid scid,
        sc.cname scname
      from (select * from tm_category where pcid=0) c
      inner join (select * from tm_category where pcid>0) sc 
      on c.cid=sc.pcid
      order by c.corder desc, c.cid, sc.corder desc, sc.cid
    ';
    $result=$this->execSQL($sql);
    $cid=-1;
    $id=-1;
    $ret=[];
    foreach($result as $val){
      if($cid!=$val["cid"]){
        $id++;
        $ret[$id]=[];
        $ret[$id]["cid"]=$val["cid"];
        $ret[$id]["cname"]=$val["cname"];
        $ret[$id]["sub"]=[];
        $cid=$val["cid"];
      }
      $sub=[];
      $sub["scid"]=$val["scid"];
      $sub["scname"]=$val["scname"];
      $ret[$id]["sub"][]=$sub;
    }
    return $ret;
    /*
      $return[0～(大カテゴリ数-1)]["cid"]
      $return[0～(大カテゴリ数-1)]["cname"]
      $return[0～(大カテゴリ数-1)]["sub"][0～(小カテゴリ数-1)]
      出力例：
      $return=>{
        [0]=>{
          "cid"=>"1", "cname"=>"大カテゴリ1",
          "sub"=>{ [0]=>{"scid"=>"1", "scname"=>"小カテゴリ1"}, [1]=>{"scid"=>"2","scname"=>"小カテゴリ2"} }
        },
        [1]=>{
          "cid"=>"2", "cname"=>"大カテゴリ2", "sub"=>{}
        [2]=>{・・・}
        }
      }
    */
  }


  //★ カテゴリ編集用 大カテゴリと小カテゴリを左結合したリストを出力する
  public function getCateLeftSubCateList(){
    $sql='
      SELECT
        c.cid AS cid,
        c.corder AS corder,
        c.cname AS cname,
        c.cdescription AS cdescription,
        sc.cid AS scid,
        sc.corder AS scorder,
        sc.cname AS scname,
        sc.cdescription AS scdescription
      FROM (SELECT * FROM tm_category WHERE pcid=0) c
      LEFT JOIN (SELECT * FROM tm_category WHERE pcid>0) sc 
      ON c.cid=sc.pcid
      ORDER BY c.corder DESC,c.cid,sc.corder desc,sc.cid
    ';
    $result=$this->execSQL($sql);
    $cid=-1;
    $id=-1;
    $ret=[];
    foreach($result as $val){
      if($cid!=$val["cid"]){
        $id++;
        $ret[$id]=[];
        $ret[$id]["cid"]=$val["cid"];
        $ret[$id]["corder"]=$val["corder"];
        $ret[$id]["cname"]=$val["cname"];
        $ret[$id]["_cname"]=html_entity_decode($val["cname"]);
        $ret[$id]["cdescription"]=$val["cdescription"];
        $ret[$id]["_cdescription"]=html_entity_decode($val["cdescription"]);
        $ret[$id]["sub"]=[];
        $cid=$val["cid"];
        if($id!=0){$ret[$id-1]["sub"][count($ret[$id-1]["sub"])-1]["tree"]="└";}
      }
      $sub=[];
      $sub["scid"]=$val["scid"];
      $sub["scorder"]=$val["scorder"];
      $sub["scname"]=$val["scname"];
      $sub["_scname"]=html_entity_decode($val["scname"]);
      $sub["scdescription"]=$val["scdescription"];
      $sub["_scdescription"]=html_entity_decode($val["scdescription"]);
      $sub["tree"]="├";
      $ret[$id]["sub"][]=$sub;
    }
    if(count($ret)>0){
      if(count($ret[$id]["sub"])>0){
        $ret[$id]["sub"][count($ret[$id]["sub"])-1]["tree"]="└";
      }
    }
    return $ret;
    /*
      $return[0～(大カテゴリ数-1)]["cid"]
      $return[0～(大カテゴリ数-1)]["corder"]
      $return[0～(大カテゴリ数-1)]["cname"]
      $return[0～(大カテゴリ数-1)]["sub"][0～(小カテゴリ数-1)]
      出力例：
      $return=>{
        [0]=>{
          "cid"=>"1", "corder"=>"100", "cname"=>"大カテゴリ1",
          "sub"=>{
            [0]=>{"scid"=>"1", "scorder"=>"100", "scname"=>"小カテゴリ１",・・・},
            [1]=>{"scid"=>"2", "scorder"=>"1", "scname"=>"小カテゴリ２",・・・},
          }
        },
        [1]=>{
          "cid"=>"2", "corder"=>"50", "cname"=>"大カテゴリ2",
          "sub"=>{}
        },
        [2]=>{・・・
        }
      }
    */
  }


  //★ カテゴリ登録更新
  public function updateInsertCategory($cid,$pcid,$corder,$cname,$cdescription){
    if($cid==0){
      $sql='insert into tm_category(pcid,corder,cname,cdescription,crdate) values(?,?,?,?,NOW())';
      $this->execSQL($sql,[$pcid,$corder,htmlentities($cname),htmlentities($cdescription)]);
    }else{
      $sql='update tm_category set pcid=?,corder=?,cname=?,cdescription=?,crdate=NOW() where cid=?';
      $this->execSQL($sql,[$pcid,$corder,htmlentities($cname),htmlentities($cdescription),$cid]);
    }
  }


  //★ 投稿一覧表示用
  //指定条件で大カテゴリ、小カテゴリ、投稿を内部結合した[ページ数,リスト]配列を出力
  function getListPost($page,$cid,$visible,$period,$title_word,$order){
    $sql="
      select
        c.cid,
        c.cname,
        sc.cid AS scid,
        sc.cname AS scname,
        p.pid,
        p.title AS title,
        p.img,
        p.visible AS visible,
        DATE_FORMAT(p.sdate,'%Y/%m/%d %H:%i') AS sdate,
        DATE_FORMAT(p.edate,'%Y/%m/%d %H:%i') AS edate
      from t_post p
      inner join (select * from tm_category where pcid>0) sc
      on p.cid=sc.cid
      inner join (select * from tm_category where pcid=0) c
      on sc.pcid=c.cid
    ";
    $prepare=[];
    $where="";
    //WHERE句の生成
    if($visible==2){
      //公開フラグ=1(公開)
      $where.=' WHERE (p.visible=1)';
    }else if($visible==3){
      //公開フラグ=0(非公開)
      $where.=' WHERE (p.visible=0)';
    }
    if($period>1){
      if($where==''){
        $where.=' WHERE ';
      }else{
        $where.=' AND ';
      }
      if($period==2){
        //公開開始日時<=現在日時<=公開終了日時 のみ
        $where.='(p.sdate<=NOW() AND (p.edate IS NULL OR p.edate>=NOW()))';
      }else if($period==3){
        //公開終了日<=現在日時 のみ
        $where.='(p.edate IS NOT NULL AND p.edate<=NOW())';
      }else if($period==4){
        //公開開始日>現在日時 のみ
        $where.='(p.sdate>NOW())';
      }
    }
    if($cid>0){
      if($where==''){
        $where.=' WHERE ';
      }else{
        $where.=' AND ';
      }
      $where.="(c.cid = ? )";
      $prepare[]=$cid;
    }
    if($title_word!=''){
      if($where==''){
        $where.=' WHERE ';
      }else{
        $where.=' AND ';
      }
      $where.="(title like ? )";
      $prepare[]='%'.$title_word.'%';
    }
    
    $sql.=$where;
    
    //レコードカウントの取得
    $result=$this->execSQL('SELECT count(tbl.pid) AS ct FROM('.$sql.') AS tbl',$prepare);
    //総ページ数計算
    $pages=ceil($result[0]['ct']/$this->postsPerPage);
    
    //表示順
    $orderby='';
    if($order==1){
      $orderby.=' ORDER BY p.sdate DESC, p.pid ';
    }else if($order==2){
      $orderby.=' ORDER BY p.sdate, p.pid ';
    }else if($order==3){
      $orderby.=' ORDER BY c.corder DESC, c.cid, sc.corder, sc.cid, p.sdate DESC';
    }
    $sql.=$orderby;
    
    //ページャー用LIMIT
    $sql.=" LIMIT ".$this->postsPerPage*($page-1).",".$this->postsPerPage;

    $result=$this->execSQL($sql,$prepare);
    for($i=0;$i<count($result);$i++){
      if($result[$i]['img']==''){$result[$i]['img']='../imgs/no-image.png';}
    }
    return [$pages,$result];
  }


  //★ 投稿登録($pid=0)又は更新($pid!=0)
  //            $scid:小カテゴリID, $sdate:表示開始日時, $edate:表示終了日時,
  //            $uid:ユーザーID, $img:バナー画像, $title:タイトル, $body:本文, $visible:表示1、非表示0
  public function updateInsertPost($pid,$scid,$sdate,$edate,$uid,$img,$title,$description,$body,$visible){
    if($edate==''){$edate=null;}else{$edate.=':59';}
    $body_text=strip_tags($body);//タグを消したテキスト
    $body_key=mb_convert_kana($body_text,'RNHVc');//全角変換
    if($pid==0){
      $sql='insert into t_post(cid,sdate,edate,uid,uuid,img,title,description,body,body_text,body_key,visible,prdate) values(?,?,?,?,?,?,?,?,?,?,?,?,NOW())';
      $this->execSQL($sql,[$scid,$sdate,$edate,$uid,$uid,$img,$title,$description,$body,$body_text,$body_key,$visible]);
    }else{
      $sql='update t_post set cid=?,sdate=?,edate=?,uuid=?,img=?,title=?,description=?,body=?,body_text=?,body_key=?,visible=? where pid=?';
      $this->execSQL($sql,[$scid,$sdate,$edate,$uid,$img,$title,$description,$body,$body_text,$body_key,$visible,$pid]);
    }
  }


  //★ 指定した投稿ID(pid)の投稿を取得する
  public function getPost($pid){
    $sql="
      SELECT
        p.pid,
        c.cid AS cid,
        p.cid AS scid,
        DATE_FORMAT(p.sdate,'%Y/%m/%d') AS sdate1,
        TIME_FORMAT(p.sdate,'%H:%i') AS sdate2,
        DATE_FORMAT(p.edate,'%Y/%m/%d') AS edate1,
        TIME_FORMAT(p.edate,'%H:%i') AS edate2,
        p.uid,
        p.img,
        p.title,
        p.description,
        p.body,
        p.visible,
        p.counter
      FROM t_post p
      INNER JOIN (select * from tm_category where pcid>0) sc
      ON sc.cid=p.cid
      INNER JOIN (select * from tm_category where pcid=0) c
      ON sc.pcid=c.cid
      WHERE p.pid=?
    ";
    $result=$this->execSQL($sql,[$pid]);
    if(count($result)==0){
      return [];
    }else{
      return $result[0];
    }
  }
}