<?php
if(count(debug_backtrace())==0){
  //このファイルがrequireやinclude経由ではなく単独で呼ばれた場合
  header('location: ../');
  exit;
}
require(__DIR__.'/cmsModelBase.php');

class cmsModelFront extends cmsModelBase{
  const NOIMAGE="./imgs/no-image.png";

  //★ 一意な大カテゴリ一覧を取得する
  //但し小カテゴリ、有効な投稿が存在するもののみ
  //できるだけ副問い合わせで絞ってからINNER JOINする（と速くなる）
  public function fGetCategory(){
    $sql='
      SELECT DISTINCT c.cid, c.cname, c.cdescription FROM (SELECT * FROM tm_category WHERE pcid=0) AS c
      INNER JOIN(
        SELECT DISTINCT sc.cid, sc.pcid FROM(
          SELECT cid,pcid FROM tm_category WHERE pcid>0
        ) AS sc
        INNER JOIN(
          SELECT DISTINCT cid FROM t_post WHERE sdate<=NOW() AND (edate IS NULL OR edate>=NOW()) AND visible=1
        ) AS p
        ON p.cid=sc.cid
      ) AS sc
      ON c.cid=sc.pcid
      ORDER BY c.corder DESC
    ';
    return $this->execSQL($sql);
  }


  //★ 指定した投稿ID(pid)の投稿を取得する
  public function fGetPost($pid){
    $sql="
      SELECT
        p.pid,
        c.cid AS cid,
        c.cname AS cname,
        sc.cid AS scid,
        sc.cname AS scname,
        DATE_FORMAT(p.sdate,'%Y/%m/%d') AS sdate1,
        TIME_FORMAT(p.sdate,'%H:%i') AS sdate2,
        DATE_FORMAT(p.edate,'%Y/%m/%d') AS edate1,
        TIME_FORMAT(p.edate,'%H:%i') AS edate2,
        p.uid,
        p.img,
        p.title,
        p.description,
        p.body,
        p.counter
      FROM t_post p
      INNER JOIN (SELECT pcid, cid, cname from tm_category where pcid>0) AS sc
      ON sc.cid=p.cid
      INNER JOIN (SELECT cid,cname FROM tm_category WHERE pcid=0) AS c
      ON sc.pcid=c.cid
      WHERE p.pid=? AND p.visible=1 AND (p.sdate<=NOW() AND (p.edate IS NULL OR p.edate>=NOW()))
    ";
    $result=$this->execSQL($sql,[$pid]);
    if(count($result)==0){
      return [];
    }else{
      if($result[0]['img']==""){$result[0]['img']=self::NOIMAGE;}
      return $result[0];
    }
  }
  //★ 指定した投稿ID(pid)の表示回数を＋１する
  public function fIncPostCounter($pid){
    $sql='
      UPDATE t_post
      SET counter=counter+1
      WHERE pid=?
    ';
    $result=$this->execSQL($sql,[$pid]);
  }


  //★ トップ画面(index.php)用 指定した大カテゴリID($cid)の投稿一覧を取得する
  //   各小カテゴリあたり最大3つ($this->postsPerSubCategory)の投稿を取得する
  public function fGetPostByCid($cid){
    //大カテゴリ、小カテゴリをINNER JOINしたカテゴリ一覧を取り出す
    $sql='
      SELECT c.cid AS cid, sc.cid AS scid, sc.cname AS scname
      FROM (SELECT * FROM tm_category WHERE pcid=0) AS c
      INNER JOIN (SELECT * FROM tm_category WHERE pcid>0) AS sc
      ON c.cid=sc.pcid
      WHERE c.cid=?
      ORDER BY sc.corder DESC, sc.cid DESC
    ';
    $sc=$this->execSQL($sql,[$cid]);
    $result=[];
    //小カテゴリに紐づく有効な投稿一覧（各小カテゴリ毎最大３つ）を取得する
    foreach($sc as $val){
      $sql='
        SELECT * FROM t_post p 
        WHERE p.visible=1 AND (p.sdate<=NOW() AND (p.edate IS NULL OR p.edate>=NOW())) AND p.cid=?
        ORDER BY p.sdate DESC, p.pid DESC
        LIMIT '.$this->postsPerSubCategory.'
      ';
      $posts=$this->execSQL($sql,[$val['scid']]);
      if(count($posts)>0){
        $res=[];
        for($i=0;$i<count($posts);$i++){
          if($posts[$i]['img']==''){$posts[$i]['img']=self::NOIMAGE;}
          $res[]=$posts[$i];
          if($i==0){
            $res[$i]['sub_start']=true;
          }else{
            $res[$i]['sub_start']=false;
          }
          if($i==(count($posts)-1)){
            $res[$i]['sub_end']=true;
          }else{
            $res[$i]['sub_end']=false;
          }
          $res[$i]['scid']=$val['scid'];
          $res[$i]['scname']=$val['scname'];
        }
        $result=array_merge($result,$res);
      }
    }
    return $result;
  }


  //★ 人気の投稿（表示回数が多いもの順）
  //  [大カテゴリ] INNER JOIN [小カテゴリ] INNER JOIN [投稿]で、有効な投稿で表示回数(t_post.counter)が多い順にfavoriteShowCount個取得する
  public function fGetPopular(){
    $sql='
      SELECT sc.cname,p.pid,p.cid,p.title,p.img
      FROM (SELECT * FROM tm_category WHERE pcid=0) AS c
      INNER JOIN (SELECT * FROM tm_category WHERE pcid>0) AS sc
      ON c.cid=sc.pcid
      INNER JOIN (
        SELECT * FROM t_post
        WHERE visible=1 AND (sdate<=NOW() AND (edate IS NULL OR edate>=NOW())) ORDER BY counter DESC LIMIT '.$this->favoriteShowCount.'
      ) p
      ON sc.cid=p.cid
      ORDER BY p.counter DESC
    ';
    $result=$this->execSQL($sql);
    for($i=0;$i<count($result);$i++){
      if($result[$i]['img']==''){$result[$i]['img']=self::NOIMAGE;}
    }
    return $result;
  }


  //★ 一意な小カテゴリ一覧を取得する
  //但し大カテゴリ、有効な投稿が存在するもののみ
  //できるだけ副問い合わせで絞ってからINNER JOINする（と速くなる）
  public function fGetSubCategory(){
    $sql='
      SELECT DISTINCT c.cid AS cid, c.cname AS cname, sc.cid AS scid, sc.cname AS scname
      FROM (SELECT cid,pcid,cname,corder FROM tm_category WHERE pcid>0) AS sc
      INNER JOIN (SELECT cid,cname FROM tm_category WHERE pcid=0) AS c
      ON sc.pcid=c.cid
      INNER JOIN (SELECT DISTINCT cid FROM t_post WHERE sdate<=NOW() AND (edate IS NULL OR edate>=NOW()) AND visible=1) AS p
      ON p.cid=sc.cid
      ORDER BY sc.corder DESC
    ';
    return $this->execSQL($sql);
  }

  //★ 指定小カテゴリIDの指定ページの投稿一覧を取得する
  function fGetPostsBySubCid($page,$scid){
    $sql="
      select
        c.cid,
        c.cname,
        sc.cid AS scid,
        sc.cname AS scname,
        p.pid,
        p.title,
        p.img
      from (SELECT pid,cid,title,img,sdate FROM t_post WHERE visible=1 AND (sdate<=NOW() AND (edate IS NULL OR edate>=NOW())) AND cid=? ) AS p
      inner join (select * from tm_category where pcid>0) sc
      on p.cid=sc.cid
      inner join (select * from tm_category where pcid=0) c
      on sc.pcid=c.cid
      order by p.sdate DESC, p.pid DESC
    ";
    //レコードカウントの取得
    $result=$this->execSQL('SELECT count(tbl.pid) AS ct FROM('.$sql.') AS tbl',[$scid]);
    $pages=ceil($result[0]['ct']/$this->postsPerPageSub);
    
    $sql.=" LIMIT ".$this->postsPerPageSub*($page-1).",".$this->postsPerPageSub;
    $result=$this->execSQL($sql,[$scid]);
    for($i=0;$i<count($result);$i++){
      if($result[$i]['img']==''){$result[$i]['img']=self::NOIMAGE;}
    }
    return [$pages,$result];
  }


  //★ カテゴリIDからカテゴリ名を取得する
  public function fGetCnameByCid($cid){
    $sql='
      SELECT
        cname,
        cdescription,
        pcid
      FROM tm_category
      WHERE cid=?
    ';
    $result=$this->execSQL($sql,[$cid]);
    $cname='';
    $cdescription='';
    $scname='';
    $scdescription='';
    if(count($result)>0){
      if($result[0]['pcid']==0){
        $cname=$result[0]['cname'];
        $cdescription=$result[0]['cdescription'];
      }else{
        $scname=$result[0]['cname'];
        $scdescription=$result[0]['cdescription'];
        $result=$this->execSQL($sql,[$result[0]['pcid']]);
        $cname=$result[0]['cname'];
        $cdescription=$result[0]['cdescription'];
      }
    }
    if(count($result)>0){
      return [$cname,$cdescription,$scname,$scdescription];
    }else{
      return ['ホーム',$this::TITLE,'',''];
    }
  }


  //★ 指定ページの指定大カテゴリ($cis)の指定キーワードの投稿一覧を取得する
  public function fSearchKeyword($page,$cid,$keyword){
    $keyword=htmlentities($keyword);
    $keyword=mb_convert_kana($keyword,'RNHVc');//全角変換
    $keyword=mb_ereg_replace(' ','　',$keyword);
    if($keyword!=''){
      $words=explode("　",$keyword,8);
    }else{
      $words=[];
    }
    $sql="
      SELECT
        c.cid,c.cname,sc.cid AS scid,sc.cname AS scname,
        p.pid,p.img,p.title,p.body_key
      FROM (SELECT * FROM tm_category where pcid=0) AS c
      INNER JOIN (SELECT * FROM tm_category where pcid>0) AS sc
      ON c.cid=sc.pcid
      INNER JOIN (SELECT * FROM t_post #WHERE1#) AS p
      ON p.cid=sc.cid
      #WHERE2#
    ";
    $param=[];
    $w="WHERE visible=1 AND (sdate<=NOW() AND (edate IS NULL OR edate>=NOW()))";
    if(count($words)>0){
      foreach($words as $word){
        $w.=" AND (body_key LIKE ? OR title LIKE ? )";
        $param[]="%".$word."%";
        $param[]="%".$word."%";
      }
    }
    $sql=str_replace("#WHERE1#",$w,$sql);
    $w="";
    if($cid!=0){
      $w=" WHERE c.cid=?";
      $param[]=$cid;
    }
    $sql=str_replace("#WHERE2#",$w,$sql);

    //カウントの取得
    $pages=ceil($this->execSQL("SELECT COUNT(*) AS ct FROM(".$sql.") AS tb",$param)[0]['ct']/$this->postsPerPageSub);

    $sql.="       ORDER BY p.sdate DESC, p.pid";
    $sql.=" LIMIT ".$this->postsPerPageSub*($page-1).",".$this->postsPerPageSub;
    $result=$this->execSQL($sql,$param);
    for($i=0;$i<count($result);$i++){
      if($result[$i]['img']==''){$result[$i]['img']=self::NOIMAGE;}
    }
    return [$pages,$result];
  }
  
}